
# q-expansion of newform 624.2.bv.e, downloaded from the LMFDB on 12 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 624
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[79, [1, 0, 0, 0]], [469, [1, 0, 0, 0]], [209, [1, 0, 0, 0]], [145, [1, 0, -1, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 1, 0], [1, 0, -2, -2], [-1, -1, -1, 0], [2, -3, -1, 3], [0, -3, 0, -1], [-4, -1, 4, 2], [-1, -3, -1, 0], [0, -3, 1, -3], [0, 2, 1, 2], [-2, 0, 4, -2], [-4, 7, 2, -7], [12, -1, -6, 1], [1, 5, -1, -10], [3, 0, -6, 3], [-3, -4, 0, 2], [0, 8, 0, 0], [4, 3, -4, -6], [14, 1, -7, -1], [-1, -3, -1, 0], [-1, 0, 2, -8], [6, -4, 0, 2], [3, 0, -6, 5], [-4, -6, 2, 6], [0, -6, 0, 0], [0, 4, -5, 4], [-17, 2, 0, -1], [0, 3, 5, 3], [2, 0, -4, 2], [10, -5, -10, 10], [0, 8, -4, 8], [-10, 4, 0, -2], [-4, -5, -4, 0], [4, 8, -4, -16], [3, 8, 3, 0], [1, 0, -2, -5], [-8, 18, 0, -9], [-2, -10, -2, 0], [4, 6, -2, -6], [-6, 6, 6, -12], [0, 11, -3, 11], [-14, -6, 0, 3], [0, -4, 0, 8], [-2, -10, 1, 10], [16, 4, -8, -4], [9, 3, -9, -6], [0, 6, 6, 6], [-8, -20, 4, 20], [3, 7, 3, 0], [8, 0, -16, 2], [-6, 16, 0, -8], [-5, 0, 10, -1], [-9, 2, -9, 0], [-10, -2, 10, 4], [0, -5, 18, -5], [0, -11, 9, -11], [-2, -2, 2, 4], [16, 8, -8, -8], [-4, -1, 4, 2], [-10, 0, 20, -5], [0, -5, -17, -5], [-13, -8, -13, 0], [-9, 0, 18, 7], [7, -10, 0, 5], [10, -4, 0, 2], [5, 0, -10, -12], [0, -20, 0, 0], [7, 16, 0, -8], [21, 7, -21, -14], [20, -2, -10, 2], [24, 1, -12, -1], [-7, 0, 14, -11], [0, 1, -13, 1], [12, 1, -12, -2], [-4, -2, 2, 2], [2, -2, 2, 0], [9, -24, 0, 12], [-6, -10, -6, 0], [-8, 15, 4, -15], [1, 16, 1, 0], [0, -2, -14, -2], [-16, 0, 32, -5], [14, 1, -7, -1], [-1, 8, 1, -16], [0, -5, 9, -5], [-26, 12, 0, -6], [-10, 6, -10, 0], [10, -10, -5, 10], [-1, 24, -1, 0], [11, 0, -22, 9], [3, 18, 0, -9], [-4, -30, 2, 30], [7, -9, 7, 0], [0, -1, -7, -1], [0, 0, 0, 32], [15, 15, -15, -30], [-2, -12, 1, 12], [12, 34, 0, -17], [0, -17, 7, -17], [10, 0, -20, 23], [-1, 6, 0, -3], [42, 6, -21, -6], [-28, -4, 28, 8], [0, 20, -4, 20], [-5, 22, 0, -11], [1, 0, -2, -2], [0, 16, 0, -16], [-8, 0, 16, -23], [6, -4, 0, 2], [0, -4, -1, -4], [-20, -12, 20, 24], [12, -1, -6, 1], [-14, 11, -14, 0], [2, 0, -4, -14], [-16, 20, -16, 0], [0, -15, 0, 30], [8, 0, 8, 0], [0, 14, -2, 14], [0, 2, 14, 2], [14, -14, -14, 28], [12, 1, -6, -1], [0, 4, 21, 4], [-42, -4, 0, 2], [8, -24, 8, 0], [10, 35, -5, -35], [-14, 20, 0, -10], [8, -27, 8, 0], [26, 2, -26, -4], [-5, -30, 0, 15], [0, 0, 0, -19], [-8, 4, 4, -4], [-12, -38, 6, 38], [0, 19, 17, 19], [0, -4, -14, -4], [-10, 6, -10, 0], [16, 30, -8, -30], [-4, -42, -4, 0], [-10, 22, -10, 0], [-34, 0, 34, 0], [0, 9, 38, 9], [-18, 0, 36, -14], [8, -6, -4, 6], [0, -24, 0, 48], [-2, 0, 4, -30], [0, -7, 0, -7], [8, 28, 8, 0], [18, 0, -36, -23], [-10, -34, 0, 17], [35, 14, 0, -7], [7, 0, -14, -5], [4, 17, 4, 0], [0, -23, 12, -23], [36, 8, 0, -4], [0, 8, 24, 8], [0, -6, -26, -6], [6, -4, 0, 2], [-6, 30, 6, -60], [-14, -15, -14, 0], [19, -28, 0, 14], [20, 0, -40, 22], [8, -28, -4, 28], [0, 24, 0, -48], [7, 0, -14, -31], [6, 14, -6, -28], [-8, -39, 4, 39], [12, 0, -24, 0], [0, 9, 7, 9], [-30, 5, 30, -10]]
